﻿using System;
using System.Runtime.InteropServices;

namespace MeinbergAPI
{
   /// <summary>
   /// Not all API calls included below are supported by any Meinberg
   /// plug-in card. Calls to functions which are not supported by a device
   /// don't do any harm, they just return an error code to the application.
   /// However, every API call to an unsupported function results in a log entry
   /// in the Windows application event log, so an application should first check
   /// whether such an API call is supported by the specific device. API calls
   /// which are not supported by each device are commented accordingly.
   /// </summary>
   public partial class Interop
   {
      #region General functions supported by all Meinberg devices

      /// <summary>
      /// Returns the number of radio clock devices detected
      /// If this function returns 0, then no devices has been detected.
      /// </summary>
      /// <returns></returns>
      [DllImport("mbgdevio.dll", EntryPoint = "mbg_find_devices", ExactSpelling = true, CharSet = CharSet.Ansi, SetLastError = true)]
      internal static extern int mbg_find_devices();

      /// <summary>
      /// Opens the device with given index and retrieve a handle to the device
      /// E.g. if n devices have been found, valid indices are 0..n-1
      /// </summary>
      /// <param name="i">The device handle as returned by <see cref="mbg_find_devices"/>.</param>
      /// <returns>The opened device's handle.</returns>
      [DllImport("mbgdevio.dll", EntryPoint = "mbg_open_device", ExactSpelling = true, CharSet = CharSet.Ansi, SetLastError = true)]
      internal static extern int mbg_open_device(int i);

      /// <summary>
      /// Closes the device and release the device handle.
      /// The device handle value will be set to 0.
      /// </summary>
      /// <param name="h">The device handle as returned by <see cref="mbg_find_devices"/>.</param>
      /// <returns>Will always return 0.</returns>
      [DllImport("mbgdevio.dll", EntryPoint = "mbg_close_device", ExactSpelling = true, CharSet = CharSet.Ansi, SetLastError = true)]
      internal static extern int mbg_close_device(ref int h);

      /// <summary>
      /// Reads date, time, and status from the device referenced by handle h.
      /// This function is supported by all Meinberg plug-in boards and returns
      /// date and time in human readable format (calendar date and time).
      /// However, the resolution of time is limited to 10 milliseconds.
      /// </summary>
      /// <param name="h">The device handle as returned by <see cref="mbg_find_devices"/>.</param>
      /// <param name="t">The device's current time.</param>
      /// <returns></returns>
      [DllImport("mbgdevio.dll", EntryPoint = "mbg_get_time", ExactSpelling = true, CharSet = CharSet.Ansi, SetLastError = true)]
      internal static extern int mbg_get_time(int h, ref PCPS_TIME t);

      #endregion

      /// <summary>
      /// The functions below read high resolution time plus status from the device
      /// referenced by handle h.
      /// These functions are NOT supported by all Meinberg plug-in boards, and the
      /// effective resolution of the HR time depends on the characteristics of the
      /// specific device.
      /// </summary>
      #region Functions for high-resolution time

      /// <summary>
      /// Checks whether the device referenced by handle h supports HR time.
      /// If it does then the value returned for f is not 0.
      /// </summary>
      /// <param name="h">The device handle as returned by <see cref="mbg_find_devices"/>.</param>
      /// <param name="f">0 if not supported. If any other value, then supported.</param>
      /// <returns></returns>
      [DllImport("mbgdevio.dll", EntryPoint = "mbg_dev_has_hr_time", ExactSpelling = true, CharSet = CharSet.Ansi, SetLastError = true)]
      internal static extern int mbg_dev_has_hr_time(int h, ref int f);

      /// <summary>
      /// Reads a HR time stamp from the board. The returned time stamp may be biased
      /// by a latency due to the IOCTL call performed by the function.
      /// </summary>
      /// <param name="h">The device handle as returned by <see cref="mbg_find_devices"/>.</param>
      /// <param name="t">The HR timestamp to set.</param>
      /// <returns></returns>
      [DllImport("mbgdevio.dll", EntryPoint = "mbg_get_hr_time", ExactSpelling = true, CharSet = CharSet.Ansi, SetLastError = true)]
      internal static extern int mbg_get_hr_time(int h, ref PCPS_HR_TIME t);

      /// <summary>
      /// Reads a HR time stamp from the board plus a performance counter value
      /// associated to that time stamp. The performance counter value can be used
      /// by the application to compensate the latency mentioned above.
      /// </summary>
      /// <param name="h">The device handle as returned by <see cref="mbg_find_devices"/>.</param>
      /// <param name="p">The performance counter to set.</param>
      /// <returns></returns>
      [DllImport("mbgdevio.dll", EntryPoint = "mbg_get_hr_time_cycles", ExactSpelling = true, CharSet = CharSet.Ansi, SetLastError = true)]
      internal static extern int mbg_get_hr_time_cycles(int h, ref PCPS_HR_TIME_CYCLES p);

      /// <summary>
      /// Reads a HR time stamp from the board and compensates for the specified latency. Return the compensated
      /// time stamp, and optionally also return the latency value in 100 nanosecond units.
      /// If the application does not need the latency value then a null pointer may be passed
      /// for parameter l.
      /// </summary>
      /// <param name="h">The device handle as returned by <see cref="mbg_find_devices"/>.</param>
      /// <param name="t">The compensated timestamp to set.</param>
      /// <param name="l">The latency. Can be nulled if not required.</param>
      /// <returns></returns>
      [DllImport("mbgdevio.dll", EntryPoint = "mbg_get_hr_time_comp", ExactSpelling = true, CharSet = CharSet.Ansi, SetLastError = true)]
      internal static extern int mbg_get_hr_time_comp(int h, ref PCPS_HR_TIME t, ref int l);

      #endregion

      /// <summary>
      /// The functions below can be used with the capture buffer available on some cards.
      /// These functions are NOT supported by all Meinberg plug-in boards, and the
      /// effective accuracy of the time capture events depends on the characteristics
      /// of the specific device.
      /// </summary>
      #region Functions for the event buffer

      /// <summary>
      /// Check whether the device referenced by handle h provides support for
      /// user capture events.
      /// If it does then the value returned for f is not 0.
      /// </summary>
      /// <param name="h">The device handle as returned by <see cref="mbg_find_devices"/>.</param>
      /// <param name="f">0 if not supported. If any other value, then supported.</param>
      /// <returns></returns>
      [DllImport("mbgdevio.dll", EntryPoint = "mbg_dev_has_ucap", ExactSpelling = true, CharSet = CharSet.Ansi, SetLastError = true)]
      internal static extern int mbg_dev_has_ucap(int h, ref int f);

      /// <summary>
      /// Clear the on-board user capture FIFO buffer
      /// </summary>
      /// <param name="h">The device handle as returned by <see cref="mbg_find_devices"/>.</param>
      /// <returns></returns>
      [DllImport("mbgdevio.dll", EntryPoint = "mbg_clr_ucap_buff", ExactSpelling = true, CharSet = CharSet.Ansi, SetLastError = true)]
      internal static extern int mbg_clr_ucap_buff(int h);

      /// <summary>
      /// Retrieve the maximum number of events that can be stored in the on-board
      /// FIFO buffer, and the number of events currently saved in the buffer.
      /// </summary>
      /// <param name="h">The device handle as returned by <see cref="mbg_find_devices"/>.</param>
      /// <param name="p">The maximum number of entries.</param>
      /// <returns></returns>
      [DllImport("mbgdevio.dll", EntryPoint = "mbg_get_ucap_entries", ExactSpelling = true, CharSet = CharSet.Ansi, SetLastError = true)]
      internal static extern int mbg_get_ucap_entries(int h, ref PCPS_UCAP_ENTRIES p);

      /// <summary>
      /// Retrieve an entry from the on-board user capture FIFO buffer. The returned entry
      /// is removed from the buffer. If no event is available in the buffer, i.e. the buffer
      /// is empty, then both the seconds and fractions of the returned time stamp are 0.
      /// </summary>
      /// <param name="h">The device handle as returned by <see cref="mbg_find_devices"/>.</param>
      /// <param name="t">The returned entry.</param>
      /// <returns></returns>
      [DllImport("mbgdevio.dll", EntryPoint = "mbg_get_ucap_event", ExactSpelling = true, CharSet = CharSet.Ansi, SetLastError = true)]
      internal static extern int mbg_get_ucap_event(int h, ref PCPS_HR_TIME t);

      #endregion

      /// <summary>
      /// The functions below can be used to read the receiver position from GPS add-in
      /// cards. This is NOT supported by non-GPS add-in cards, so there's another
      /// function which checks whether a specific device is a GPS receiver.
      /// </summary>
      #region Functions for position
      /// <summary>
      /// Check whether the device referenced by handle h is a GPS receiver.
      /// If it is then the value returned for f is not 0.
      /// </summary>
      /// <param name="h">The device handle as returned by <see cref="mbg_find_devices"/>.</param>
      /// <param name="f">0 if not supported. If any other value, then supported.</param>
      /// <returns></returns>
      [DllImport("mbgdevio.dll", EntryPoint = "mbg_dev_is_gps", ExactSpelling = true, CharSet = CharSet.Ansi, SetLastError = true)]
      internal static extern int mbg_dev_is_gps(int h, ref int f);

      /// <summary>
      /// Read the current receiver position.
      /// </summary>
      /// <param name="h">The device handle as returned by <see cref="mbg_find_devices"/>.</param>
      /// <param name="p">The receiver's current position.</param>
      /// <returns></returns>
      [DllImport("mbgdevio.dll", EntryPoint = "mbg_get_gps_pos", ExactSpelling = true, CharSet = CharSet.Ansi, SetLastError = true)]
      internal static extern int mbg_get_gps_pos(int h, ref MBG_RCVR_POS p);
      #endregion
      
      #region Service Functions

      /// <summary>
      /// Returns the version number of the DLL
      /// <returns>The version number.</returns>
      /// </summary>
      [DllImport("mbgsvcio.dll", EntryPoint = "mbg_get_gps_pos", ExactSpelling = true, CharSet = CharSet.Ansi, SetLastError = true)]
      internal static extern int mbgsvcio_get_version();

      /// <summary>
      /// Checks that the DLL interface specification version is compatible with the
      /// DLL version. Pass the MBGSVCIO_VERSION code from <see cref="mbgsvcio_get_version"/> as parameter.
      /// <returns>PCPS_SUCCESS on success.</returns>
      /// </summary>
      [DllImport("mbgsvcio.dll", EntryPoint = "mbgsvcio_check_version", ExactSpelling = true, CharSet = CharSet.Ansi, SetLastError = true)]
      internal static extern int mbgsvcio_check_version();

      /// <summary>
      /// Checks if the Meinberg time adjustment service is active.
      /// <returns>a positive value if the service is active
      ///          0 if the service is not active</returns>
      [DllImport("mbgsvcio.dll", EntryPoint = "mbg_time_adjustment_active", ExactSpelling = true, CharSet = CharSet.Ansi, SetLastError = true)]
      internal static extern int mbg_time_adjustment_active();

      /// <summary>
      /// Checks if the ref time source (e.g. radio clock) used by the Meinberg
      /// time adjustment service is accessible. This should be always the case
      /// with PCI plug-in boards, but this can also fail with an external clock
      /// if for example the serial connection is broken or the device is powered
      /// off.
      /// <returns>a positive value if the ref time source is accessible
      ///          0 if the ref time source is not accessible
      ///          a negative number if the status could not be determined.</returns>
      ///          </summary>
      [DllImport("mbgsvcio.dll", EntryPoint = "mbg_ref_time_accessible", ExactSpelling = true, CharSet = CharSet.Ansi, SetLastError = true)]
      internal static extern int mbg_ref_time_accessible();

      /// <summary>
      /// Queries the status of the ref time source (e.g. radio clock) used by the
      /// Meinberg time adjustment service.
      /// <returns>a positive value if the ref time source is accessible
      ///          0 if the ref time source is not accessible
      ///          a negative number if the status could not be determined.</returns>
      /// </summary>
      [DllImport("mbgsvcio.dll", EntryPoint = "mbg_get_ref_time_status", ExactSpelling = true, CharSet = CharSet.Ansi, SetLastError = true)]
      internal static extern int mbg_get_ref_time_status();

      #endregion
   }
}
