﻿using System;

namespace MeinbergAPI
{
   public class MeinbergDevice
   {
      #region Private Properties

      private int _DeviceHandle;
      private bool _SupportsHRTime;
      private bool _SupportsUserCaptureEvents;
      private bool _IsGpsReceiver;

      #endregion

      #region Public Properties

      /// <summary>
      /// The device's handle.
      /// </summary>
      public int DeviceHandle
      {
         get { return _DeviceHandle; }
         set { _DeviceHandle = value; }
      }

      /// <summary>
      /// Specifies whether the device supports HR time.
      /// </summary>
      public bool SupportsHRTime
      {
         get { return _SupportsHRTime; }
         set { _SupportsHRTime = value; }
      }

      /// <summary>
      /// Specified whether the device supports User Capture Events.
      /// </summary>
      public bool SupportsUserCaptureEvents
      {
         get { return _SupportsUserCaptureEvents; }
         set
         {
            _SupportsUserCaptureEvents = value;
         }
      }

      /// <summary>
      /// Specifies whether the device is a GPS receiver.
      /// </summary>
      public bool IsGpsReceiver
      {
         get { return _IsGpsReceiver; }
         set
         {
            _IsGpsReceiver = value;
         }
      }

      public bool CapabilitiesChecked = false;

      #endregion

      #region Constructors & Destructors

      /// <summary>
      /// Initializes an instance of the MeinbergDevice class.
      /// </summary>
      /// <param name="deviceHandle">The device's handle.</param>
      public MeinbergDevice(int deviceHandle)
      {
         _DeviceHandle = deviceHandle;
      }

      /// <summary>
      /// Initializes an instance of the MeinbergDevice class.
      /// </summary>
      /// <param name="deviceHandle">The device's handle.</param>
      /// <param name="initializeCapabilities">Specify whether to initialize the class with the device's capabilities.</param>
      public MeinbergDevice(int deviceHandle, bool initializeCapabilities)
      {
         _DeviceHandle = deviceHandle;
         if (initializeCapabilities) CheckCapabilities();
      }

      #endregion

      #region Public Methods

      public void CheckCapabilities()
      {
         int supportsHRTime = -1;
         Interop.mbg_dev_has_hr_time(DeviceHandle, ref supportsHRTime);
         SupportsHRTime = supportsHRTime != 0;

         int supportsUCETime = -1;
         Interop.mbg_dev_has_ucap(DeviceHandle, ref supportsUCETime);
         SupportsUserCaptureEvents = supportsUCETime != 0;

         int supportsGps = -1;
         Interop.mbg_dev_is_gps(DeviceHandle, ref supportsGps);
         IsGpsReceiver = supportsGps != 0;

         CapabilitiesChecked = true;

      }

      public bool Open()
      {
         return Interop.mbg_open_device(this._DeviceHandle) == this._DeviceHandle;
      }

      public bool Close()
      {
         return Interop.mbg_close_device(ref this._DeviceHandle) == 0;
      }

      /// <summary>
      /// Gets the time from the device.
      /// </summary>
      /// <param name="successful">Specifies whether the operation was successful.</param>
      /// <returns></returns>
      public Interop.PCPS_TIME GetTime(out bool successful)
      {
         var time = new Interop.PCPS_TIME();
         successful = Interop.mbg_get_time(this._DeviceHandle, ref time) == Interop.PCPS_SUCCESS;
         if (successful)
            return time;
         else
            return new Interop.PCPS_TIME();
      }

      /// <summary>
      /// Gets high-resolution time from the device.
      /// </summary>
      /// <param name="successful">Specifies whether the operation was successful.</param>
      /// <returns>Returns the time.</returns>
      public Interop.PCPS_HR_TIME GetHRTime(out bool successful)
      {
         if (CapabilitiesChecked)
            throw new Exception("Before accessing extended functions, you first have to check the device's capabilities by calling CheckCapabilities.");

         if (!SupportsHRTime)
            throw new NotSupportedException("This device does not support high-resolution time.");

         var time = new Interop.PCPS_HR_TIME();
         successful = Interop.mbg_get_hr_time(this._DeviceHandle, ref time) == Interop.PCPS_SUCCESS;
         if (successful)
            return time;
         else
            return new Interop.PCPS_HR_TIME();
      }

      /// <summary>
      /// Reads a HR time stamp from the board plus a performance counter value
      /// associated to that time stamp.
      /// </summary>
      /// <param name="successful">Specifies whether the operation was successful.</param>
      /// <returns>Returns the high-resolution time.</returns>
      public Interop.PCPS_HR_TIME_CYCLES GetHRTimeCycles(out bool successful)
      {
         if (CapabilitiesChecked)
            throw new Exception("Before accessing extended functions, you first have to check the device's capabilities by calling CheckCapabilities.");

         if (!SupportsHRTime)
            throw new NotSupportedException("This device does not support high-resolution time.");

         var time = new Interop.PCPS_HR_TIME_CYCLES();
         successful = Interop.mbg_get_hr_time_cycles(this._DeviceHandle, ref time) == Interop.PCPS_SUCCESS;
         if (successful)
            return time;
         else
            return new Interop.PCPS_HR_TIME_CYCLES();
      }

      /// <summary>
      /// Reads a HR time stamp from the board and compensates for the specified latency.
      /// </summary>
      /// <param name="successful">Specifies whether the operation was successful.</param>
      /// <param name="latency">The latency to compensate for (Nullable)</param>
      /// <returns>Returns the compensated time stamp, and optionally also return the latency 
      /// value in 100 nanosecond units.</returns>
      public Interop.PCPS_HR_TIME GetHRTimeCycles(out bool successful, out int latency)
      {
         if (CapabilitiesChecked)
            throw new Exception("Before accessing extended functions, you first have to check the device's capabilities by calling CheckCapabilities.");

         if (!SupportsHRTime)
            throw new NotSupportedException("This device does not support high-resolution time.");

         var time = new Interop.PCPS_HR_TIME();
         int l = -1;

         successful = Interop.mbg_get_hr_time_comp(this._DeviceHandle, ref time, ref l) == Interop.PCPS_SUCCESS;
         if (successful)
         {
            latency = l;
            return time;
         }
         else
         {
            latency = -1;
            return new Interop.PCPS_HR_TIME();
         }
      }

      /// <summary>
      /// 
      /// </summary>
      /// <returns>Returns whether the operation was successful.</returns>
      public bool ClearOnboardBuffers()
      {
         if (CapabilitiesChecked)
            throw new Exception("Before accessing extended functions, you first have to check the device's capabilities by calling CheckCapabilities.");

         if (!SupportsHRTime)
            throw new NotSupportedException("This device does not support user events.");

         return Interop.mbg_clr_ucap_buff(this._DeviceHandle) == Interop.PCPS_SUCCESS;
      }

      /// <summary>
      /// Retrieve the maximum number of events that can be stored in the on-board FIFO buffer, 
      /// and the number of events currently saved in the buffer.
      /// </summary>
      /// <param name="successful">Specifies whether the operation was successful.</param>
      /// <returns>Returns the entries in the buffer. </returns>
      public Interop.PCPS_UCAP_ENTRIES GetEvents(out bool successful)
      {
         if (CapabilitiesChecked)
            throw new Exception("Before accessing extended functions, you first have to check the device's capabilities by calling CheckCapabilities.");

         if (!SupportsHRTime)
            throw new NotSupportedException("This device does not support user events.");

         var allEvents = new Interop.PCPS_UCAP_ENTRIES();
         successful = Interop.mbg_get_ucap_entries(this._DeviceHandle, ref allEvents) == Interop.PCPS_SUCCESS;
         if (successful)
            return allEvents;
         else
            return new Interop.PCPS_UCAP_ENTRIES();
      }

      /// <summary>
      /// Retrieve an entry from the on-board user capture FIFO buffer. The returned entry
      /// is removed from the buffer. 
      /// </summary>
      /// <param name="successful">Specifies whether the operation was successful.</param>
      /// <returns>Returns the entry requested. If no event is available in the buffer, i.e. the buffer
      /// is empty, then both the seconds and fractions of the returned time stamp are 0.</returns>
      public Interop.PCPS_HR_TIME GetEvent(out bool successful)
      {
         if (CapabilitiesChecked)
            throw new Exception("Before accessing extended functions, you first have to check the device's capabilities by calling CheckCapabilities.");

         if (!SupportsHRTime)
            throw new NotSupportedException("This device does not support user events.");

         var singleEvent = new Interop.PCPS_HR_TIME();
         successful = Interop.mbg_get_ucap_event(this._DeviceHandle, ref singleEvent) == Interop.PCPS_SUCCESS;
         if (successful)
            return singleEvent;
         else
            return new Interop.PCPS_HR_TIME();
      }

      /// <summary>
      /// Gets the device's current GPS position.
      /// </summary>
      /// <param name="successful">Specifies whether the operation was successful.</param>
      /// <returns>Returns the GPS position.</returns>
      public Interop.MBG_RCVR_POS GetDeviceGPSPosition(out bool successful)
      {
         if (CapabilitiesChecked)
            throw new Exception("Before accessing extended functions, you first have to check the device's capabilities by calling CheckCapabilities.");

         if (!SupportsHRTime)
            throw new NotSupportedException("This device does not support GPS functionality.");

         var gpsPosition = new Interop.MBG_RCVR_POS();
         successful = Interop.mbg_get_gps_pos(this._DeviceHandle, ref gpsPosition) == Interop.PCPS_SUCCESS;
         if (successful)
            return gpsPosition;
         else
            return new Interop.MBG_RCVR_POS();
      }

      #endregion

   }
}
