﻿using System;
using System.Runtime.InteropServices;

namespace MeinbergAPI
{
   public partial class Interop
   {
      #region Time Structures

      /// <summary>
      /// The DLL version required by this interface specification
      /// </summary>
      public const short MBGDEVIO_VERSION = 0x212;

      /// <summary>
      /// This structure is passed to the function mbg_get_time to read date, time, and status       
      /// </summary>
      [StructLayout(LayoutKind.Sequential, Pack = 1)]
      public struct PCPS_TIME
      {
         // 0..99
         public byte sec100;
         // 0..59, 60 if leap second
         public byte sec;
         // 0..59
         public byte min;
         // 0..23
         public byte hour;
         // 1..31
         public byte mday;
         // 1..7, 1 = Monday
         public byte wday;
         // 1..12
         public byte month;
         // 0..99 (year modulo 100)
         public byte year;
         // flag bits, see below
         public byte status;
         // 0..255
         public byte signal;
         // [h] signed!! (UTC = t - offs_utc)
         public byte offs_utc;
      }

      /// <summary>
      /// This structure has been introduced to be able to handle high resolution time stamps.
      /// </summary>
      [StructLayout(LayoutKind.Sequential, Pack = 1)]
      public struct PCPS_TIME_STAMP
      {
         // UInt32, seconds since 1970 (UTC)
         public int sec;
         // UInt32, fractions of second ( &HFFFFFFFF == 0.9999.. sec)
         public int frac;
      }

      /// <summary>
      /// This structure holds a HR time stamp, plus UTC offset and status info
      /// </summary>
      [StructLayout(LayoutKind.Sequential, Pack = 1)]
      public struct PCPS_HR_TIME
      {
         public PCPS_TIME_STAMP tstamp;
         // [sec] signed!! (local time = tstamp + offs_utc)
         public int utc_offs;
         // flag bits, see below
         public short status;
         // 0..255
         public byte signal;
      }

      /// <summary>
      /// This structure holds a HR time plus an associated Windows
      /// performance counter value which can be used to relate performance
      /// counter values to the current time. Please note that an application
      /// must also determine the Windows performance counter frequency in order
      /// to convert differences of performance counter values to time intervals.  
      /// </summary>
      [StructLayout(LayoutKind.Sequential, Pack = 1)]
      public struct PCPS_HR_TIME_CYCLES
      {
         // UInt64, performance counter value
         public long cycles;
         // associated HR time
         public PCPS_HR_TIME t;
      }

      /// <summary>
      /// This structure contains the maximum number of entries of an on-board
      /// user capture FIFO buffer plus the number of currently used entries.
      /// </summary>
      [StructLayout(LayoutKind.Sequential, Pack = 1)]
      public struct PCPS_UCAP_ENTRIES
      {
         // the number of saved capture events
         public int used;
         // capture buffer size
         public int max;
      }

      #endregion

      #region Structures for Geographic Position

      /// <summary>
      /// This type is used to hold a geographic position in cartesian coordinates.
      /// </summary>
      [StructLayout(LayoutKind.Sequential, Pack = 1)]
      public struct MBG_POS_XYZ
      {
         // x coordinate, in meters
         public double x;
         // y coordinate, in meters
         public double y;
         // z coordinate, in maters
         public double z;
      }

      /// <summary>
      /// This type is used to pass a geographic position in raw geographic coordinates.
      /// </summary>
      [StructLayout(LayoutKind.Sequential, Pack = 1)]
      public struct MBG_POS_LLA
      {
         // latitude, in radians
         public double lat;
         // longitude, in radians
         public double lon;
         // altitude (height above ellipsoid), in meters
         public double alt;
      }

      /// <summary>
      /// This type is used in conjuction with <see cref="MBG_RCVR_POS" /> to aid in 
      /// storing the receiver position in a human readable format
      /// </summary>
      [StructLayout(LayoutKind.Sequential, Pack = 1)]
      public struct MBG_DMS
      {
         // ASCII code of 'N', 'E', 'S' or 'W'
         public short prefix;
         // [0...90 (lat) or 0...180 (lon)]
         public short deg;
         // [0...59]
         public short min;
         // [0...59.999]
         public double sec;
      }

      /// <summary>
      /// This type is used to store a geographic coordinate in broken-down (human readable) format.
      /// </summary>
      [StructLayout(LayoutKind.Sequential, Pack = 1)]
      public struct MBG_RCVR_POS
      {
         // always WGS84 ECEF coordinates
         public MBG_POS_XYZ xyz;
         // depending on the ellipsoid used for reference
         public MBG_POS_LLA lla;
         // longitude in degrees, minutes, seconds
         public MBG_DMS longitude;
         // latitude in degrees, minutes, seconds
         public MBG_DMS latitude;
         // ellipsoid used for reference
         public short ellipsoid;
      }

      #endregion
   }
}
